# -*- coding: utf-8 -*-
import pytest
from reportserver.lib.db import sqlfactory


class TestSQLFactoy(object):
    def test_sqlfactory_without_any_konfiguration(self):
        factory = sqlfactory.SQLFactory()
        assert factory is not None
        with pytest.raises(ValueError):
            factory.create()

    def test_sqlfactory_with_one_indicator(self):
        factory = sqlfactory.SQLFactory("eval1")
        factory.indicators = ["CO01"]
        sql = factory.create().sql
        expected = "SELECT MAX(CASE WHEN gender=0 THEN 'female' ELSE 'male' END) as gender, " \
                   "COUNT(gender) as total, " \
                   "SUM(CASE WHEN start_date is not null THEN 1 ELSE 0 END) AS enters, " \
                   "SUM(CASE WHEN (start_date is not null and end_date is not null) THEN 1 ELSE 0 END) AS exits, " \
                   "SUM(CASE WHEN CO01 = TRUE THEN 1 ELSE 0 END) AS CO01 FROM eval1 group by gender"
        assert(sql == expected)

    def test_sqlfactory_with_two_indicators(self):
        factory = sqlfactory.SQLFactory("eval1")
        factory.indicators = ["CO01", "CR01"]
        sql = factory.create().sql
        expected = "SELECT MAX(CASE WHEN gender=0 THEN 'female' ELSE 'male' END) as gender, " \
                   "COUNT(gender) as total, " \
                   "SUM(CASE WHEN start_date is not null THEN 1 ELSE 0 END) AS enters, " \
                   "SUM(CASE WHEN (start_date is not null and end_date is not null) THEN 1 ELSE 0 END) AS exits, " \
                   "SUM(CASE WHEN CO01 = TRUE THEN 1 ELSE 0 END) AS CO01, " \
                   "SUM(CASE WHEN CR01 = TRUE THEN 1 ELSE 0 END) AS CR01 " \
                   "FROM eval1 group by gender"
        assert(sql == expected)

    def test_sqlfactory_with_one_filter(self):
        factory = sqlfactory.SQLFactory("eval1")
        factory.indicators = ["CO01"]
        factory.filters = [("start_date >= %s", "2015-01-01")]
        sql = factory.create().sql
        expected = "SELECT MAX(CASE WHEN gender=0 THEN 'female' ELSE 'male' END) as gender, " \
                   "COUNT(gender) as total, " \
                   "SUM(CASE WHEN start_date is not null THEN 1 ELSE 0 END) AS enters, " \
                   "SUM(CASE WHEN (start_date is not null and end_date is not null) THEN 1 ELSE 0 END) AS exits, " \
                   "SUM(CASE WHEN CO01 = TRUE THEN 1 ELSE 0 END) AS CO01 FROM eval1 " \
                   "WHERE start_date >= %s "\
                   "group by gender"
        assert(sql == expected)

    def test_sqlfactory_with_two_filters(self):
        factory = sqlfactory.SQLFactory("eval1")
        factory.indicators = ["CO01"]
        factory.filters = [("start_date >= %s", "2015-01-01"),("end_date <= %s", "2017-01-01")]
        sql = factory.create().sql
        expected = "SELECT MAX(CASE WHEN gender=0 THEN 'female' ELSE 'male' END) as gender, " \
                   "COUNT(gender) as total, " \
                   "SUM(CASE WHEN start_date is not null THEN 1 ELSE 0 END) AS enters, " \
                   "SUM(CASE WHEN (start_date is not null and end_date is not null) THEN 1 ELSE 0 END) AS exits, " \
                   "SUM(CASE WHEN CO01 = TRUE THEN 1 ELSE 0 END) AS CO01 FROM eval1 " \
                   "WHERE start_date >= %s "\
                   "AND end_date <= %s "\
                   "group by gender"
        assert(sql == expected)

if __name__ == "__main__":
    unittest.main()
